@@ -34,7 +34,7 @@
 #include "vstr.h"
 #include "config.h"
 
-static char gp_title[] = "dvdspanky 1.0.6 - (C) 2007 Jeffrey Grembecki\n\n";
+static char gp_title[] = "dvdspanky 1.0.7 - (c) 2007 Jeffrey Grembecki\n\n";
 static char gp_licence[] =
 "This program is free software; you can redistribute it and/or modify\n"
 "it under the terms of the GNU General Public License as published by\n"
@@ -120,18 +120,21 @@ static struct option gp_longopts[] =
 	{"vrate", 1, 0, 'r'},
 	{"arate", 1, 0, 'R'},
 	{"size", 1, 0, 's'},
+	{"speed", 0, 0, 'S'},
 	{"postprocess", 0, 0, 'T'},
 	{"vcrop", 1, 0, 'V'},
 	{"verbose", 0, 0, 'v'},
 };
 
-static char gp_shortopts[] = "012a:AbBc:f:Fg:G:hH:i:ILmMnNo:OpPr:R:s:TvV:";
+static char gp_shortopts[] = "012a:AbBc:f:Fg:G:hH:i:ILmMnNo:OpPr:R:s:STvV:";
 
 static pcre *gp_idregex = NULL;
 static pcre *gp_transregex = NULL;
 static pcre *gp_tcvideoregex = NULL;
 static pcre *gp_tcaudioregex = NULL;
 static pcre *gp_warnerrregex = NULL;
+static pcre *gp_soxregex = NULL;
+static pcre *gp_ffmpegregex = NULL;
 
 #define ML_ERROR 4
 #define ML_WARNING 3
@@ -205,69 +208,86 @@ static void cleanup()
 /* encode / preview */
 int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 {
-	vstr *p_generic, *p_video, *p_videoextra, *p_audio, *p_final;
+	vstr *p_generic, *p_video, *p_videoextra, *p_audio, *p_final, *p_outcodec, *p_preview;
 
 	p_generic = vs_new();
 	p_video = vs_new();
 	p_videoextra = vs_new();
 	p_audio = vs_new();
 	p_final = vs_new();
+	p_outcodec = vs_new();
+	p_preview = vs_new();
 
-	if(!p_generic || !p_video || !p_videoextra || !p_audio || !p_final)
+	if(!p_generic || !p_video || !p_videoextra || !p_audio || !p_final || !p_outcodec || !p_preview)
 	{
 		if(p_generic) vs_free(p_generic);
 		if(p_video) vs_free(p_video);
 		if(p_videoextra) vs_free(p_videoextra);
 		if(p_audio) vs_free(p_audio);
 		if(p_final) vs_free(p_final);
+		if(p_outcodec) vs_free(p_outcodec);
+		if(p_preview) vs_free(p_preview);
 		printerror("unable to allocate strings");
 		return 1;
 	}
 
 #define freestrings() { vs_free(p_generic); vs_free(p_video); vs_free(p_videoextra); \
-	vs_free(p_audio); vs_free(p_final); }
+	vs_free(p_audio); vs_free(p_final); vs_free(p_preview);}
 
 	/* generic */
-	vs_cat(p_generic, "transcode -i %s", p_in->p_filename);
+	vs_cat(p_generic, "transcode\n-i\n%s", p_in->p_filename);
 	if(!(options & SPANK_NONICE))
-		vs_cat(p_generic, " --nice 19");
+		vs_cat(p_generic, "\n--nice\n19");
 	if(options & SPANK_MPLAYER)
 	{
 		unlink("stream.yuv");
-		vs_cat(p_generic, " -H 0 -x mplayer,mplayer -g %dx%d -n 0x1 -e %d,%d,%d", p_in->width,
+		vs_cat(p_generic, "\n-H\n0\n-x\nmplayer,mplayer\n-g\n%dx%d\n-n\n0x1\n-e\n%d,%d,%d", p_in->width,
 			p_in->height, p_in->hz,	(p_in->abits) ? p_in->abits : 16, p_in->achans);
 	}
 
 	/* audio */
-	vs_cat(p_audio, " -E 48000,%d,%d -N 0x2000 -b %d",
+	vs_cat(p_audio, "\n-E\n48000,%d,%d\n-N\n0x2000\n-b\n%d",
 		(p_out->abits) ? p_out->abits : 16, p_out->achans, p_out->arate);
 
 	/* video */
 	{
 		int asr = (p_out->aspect < 1.5) ? 2 : 3;
-		vs_cat(p_video, " -w %d -f %.3f --export_fps %.3f --import_asr %d --export_asr %d"
-			" --encode_fields %c", 	p_out->vrate, p_in->fps, p_out->fps, asr, asr,
+		vs_cat(p_video, "\n-w\n%d\n-f\n%.3f\n--export_fps\n%.3f\n--import_asr\n%d\n--export_asr\n%d"
+			"\n--encode_fields\n%c", 	p_out->vrate, p_in->fps, p_out->fps, asr, asr,
 			(p_out->p_vcodec[0] == 'P') ? 't' : 'b');
 	}
 
 	/* video extra */
 	if(options & SPANK_DOUBLEFPS)
-		vs_cat(p_videoextra, " -J doublefps");
-	if(options & SPANK_NEWFPS)
-		vs_cat(p_videoextra, " -J modfps=clonetype=1");
+		vs_cat(p_videoextra, "\n-J\ndoublefps");
+	if(options & SPANK_HARDFPS)
+		vs_cat(p_videoextra, "\n-J\nmodfps=clonetype=1");
 	if(options & SPANK_FIXNTSC)
-		vs_cat(p_videoextra, " -M 2 -J ivtc,32detect=force_mode=5,decimate --hard_fps");
+		vs_cat(p_videoextra, "\n-M\n2\n-J\nivtc,32detect=force_mode=5,decimate\n--hard_fps");
 	if(p_out->vcrop || p_out->hcrop)
-		vs_cat(p_videoextra, " -j %d,%d,%d,%d",
+		vs_cat(p_videoextra, "\n-j\n%d,%d,%d,%d",
 			p_out->vcrop, p_out->hcrop, p_out->vcrop, p_out->hcrop);
 	if(p_out->gamma != 1.0)
-		vs_cat(p_videoextra, " -G %f", p_out->gamma);
+		vs_cat(p_videoextra, "\n-G\n%f", p_out->gamma);
 	if(options & SPANK_PPROCESS)
-		vs_cat(p_videoextra, " -J pp=de");
+		vs_cat(p_videoextra, "\n-J\npp=de");
 	if(options & SPANK_NORMALISE)
-		vs_cat(p_videoextra, " -J normalize=algo=2 -s 0.5");
+		vs_cat(p_videoextra, "\n-J\nnormalize=algo=2\n-s\n0.5");
 	if(p_filters[0])
-		vs_cat(p_videoextra, " -J %s", p_filters);
+		vs_cat(p_videoextra, "\n-J\n%s", p_filters);
+
+	/* preview options */
+	if(p_out->vcrop || p_out->hcrop)
+		vs_cat(p_preview, "\n-j\n%d,%d,%d,%d",
+			p_out->vcrop, p_out->hcrop, p_out->vcrop, p_out->hcrop);
+	if(p_out->gamma != 1.0)
+		vs_cat(p_preview, "\n-G\n%f", p_out->gamma);
+
+	/* export codecs */
+	if(options & SPANK_SPEED)
+		vs_cat(p_outcodec, "\n-y\nffmpeg,wav\n-F\nmpeg2video\n-m\n%s_.wav", p_out->p_filename);
+	else
+		vs_cat(p_outcodec, "\n-y\nffmpeg\n-F\nmpeg2video\n-m\n%s.ac3", p_out->p_filename);
 
 	/* create ffmpeg.cfg */
 	system("echo \"[mpeg2video]\nvrc_minrate=0\nvrc_maxrate=7500\nvrc_buf_size=1792\n\""
@@ -280,9 +300,9 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		int count;
 
 		print(ML_INFO, NULL, "generating previews.."); /* generate previews */
-		if(forkitf(NULL, "transcode", TRANSCODE_BIN, "%s %s -y jpg,null -F 100 -Z %dx%d"
-			" -c 300-301,600-601,900-901,1200-1201,1500-1501 -o pre",
-			p_generic->s, p_video->s,
+		if(forkitf(NULL, "transcode", TRANSCODE_BIN, "%s%s%s\n-y\njpg,null\n-F\n100\n-Z\n%dx%d"
+			"\n-c\n300-301,600-601,900-901,1200-1201,1500-1501\n-o\npre",
+			p_generic->s, p_video->s, p_preview->s,
 			(int)((double)p_out->height * p_out->aspect), p_out->height, p_out->p_filename))
 		{
 			printerror("failed generating previews");
@@ -301,7 +321,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 			while(count--)
 			{
 				if(!fnmatch("pre*.jpg", pp_dirent[count]->d_name, 0))
-					vs_cat(p_final, " %s", pp_dirent[count]->d_name);
+					vs_cat(p_final, "\n%s", pp_dirent[count]->d_name);
 			}
 			forkit(NULL, "feh", FEH_BIN, p_final->s);
 
@@ -324,7 +344,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		print(ML_INFO, NULL, "scanning volume..");
 		vs_print(p_final, "%s-tmp.ac3", p_out->p_filename);
 		if(forkitf(cb_transcode, (void*)p_out, TRANSCODE_BIN,
-			"%s %s -y null,ac3 -m %s-tmp -J astat=%s.norm",
+			"%s%s\n-y\nnull,ac3\n-m\n%s-tmp\n-J\nastat=%s.norm",
 			p_generic->s, p_audio->s, p_out->p_filename, p_out->p_filename))
 		{
 			print(ML_INFO, NULL, "");
@@ -343,13 +363,13 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 	{
 		print(ML_INFO, NULL, "starting 1st VBR pass..");
 
-		vs_print(p_final, "%s %s %s", p_generic->s, p_video->s, p_audio->s);
+		vs_print(p_final, "%s%s%s", p_generic->s, p_video->s, p_audio->s);
 		if(options & SPANK_AVOLUME)
 			vs_cat(p_final,
-				" -y ffmpeg -F mpeg2video -R 1,%s.pass1 -m %s.tmp -J astat=%s.norm",
+				"\n-y\nffmpeg\n-F\nmpeg2video\n-R\n1,%s.pass1\n-m\n%s.tmp\n-J\nastat=%s.norm",
 				p_out->p_filename, p_out->p_filename, p_out->p_filename);
 		else
-			vs_cat(p_final, " -y ffmpeg,null -F mpeg2video -R 1,%s.pass1", p_out->p_filename);
+			vs_cat(p_final, "\n-y\nffmpeg,null\n-F\nmpeg2video\n-R\n1,%s.pass1", p_out->p_filename);
 
 		if(forkit(cb_transcode, (void*)p_out, TRANSCODE_BIN, p_final->s))
 		{
@@ -413,11 +433,11 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		/* transcode */
 		print(ML_INFO, NULL, "starting 2nd VBR pass..");
 		vs_print(p_final,
-			"%s %s %s %s -Z %dx%d -y ffmpeg -F mpeg2video -R 2,%s.pass1 -o %s -m %s.ac3",
-			p_generic->s, p_audio->s, p_video->s, p_videoextra->s,
-			p_out->width, p_out->height, p_out->p_filename, p_out->p_filename, p_out->p_filename);
+			"%s%s%s%s%s\n-Z\n%dx%d\n-R\n2,%s.pass1\n-o\n%s",
+			p_generic->s, p_audio->s, p_video->s, p_videoextra->s, p_outcodec->s,
+			p_out->width, p_out->height, p_out->p_filename, p_out->p_filename);
 		if(p_out->volume != 1.0)
-			vs_cat(p_final, " -s %f", p_out->volume);
+			vs_cat(p_final, "\n-s\n%f", p_out->volume);
 		if(forkitf(cb_transcode, (void*)p_out, TRANSCODE_BIN, p_final->s))
 		{
 			print(ML_INFO, NULL, "");
@@ -439,7 +459,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 
 			print(ML_INFO, NULL, "scanning volume..");
 			if(forkitf(cb_transcode, (void*)p_out, TRANSCODE_BIN,
-				"%s %s -y null,ac3 -m %s-tmp -J astat=%s.norm",
+				"%s%s\n-y\nnull,ac3\n-m\n%s-tmp\n-J\nastat=%s.norm",
 				p_generic->s, p_audio->s, p_out->p_filename, p_out->p_filename))
 			{
 				print(ML_INFO, NULL, "");
@@ -471,11 +491,11 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 
 		/* execute */
 		print(ML_INFO, NULL, "starting CBR encode..");
-		vs_print(p_final, "%s %s %s %s -Z %dx%d -y ffmpeg -F mpeg2video -o %s -m %s.ac3",
-			p_generic->s, p_video->s, p_videoextra->s, p_audio->s,
-			p_out->width, p_out->height, p_out->p_filename, p_out->p_filename);
+		vs_print(p_final, "%s%s%s%s%s\n-Z\n%dx%d\n-o\n%s",
+			p_generic->s, p_video->s, p_videoextra->s, p_audio->s, p_outcodec->s,
+			p_out->width, p_out->height, p_out->p_filename);
 		if(p_out->volume != 1.0)
-			vs_cat(p_final, " -s %f", p_out->volume);
+			vs_cat(p_final, "\n-s\n%f", p_out->volume);
 		if(forkit(cb_transcode, (void*)p_out, TRANSCODE_BIN, p_final->s))
 		{
 			print(ML_INFO, NULL, "");
@@ -486,15 +506,45 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		print(ML_INFO, NULL, "");
 	}
 
-	/* multiplex */
+	if(options & SPANK_MPLAYER)
+		unlink("stream.yuv");
+
+	/* multiplex / audioconvert */
 	if((options & SPANK_SECOND) || !(options & (SPANK_FIRST | SPANK_SECOND)) ||
 		(options & SPANK_MULTIPLEX))
 	{
+		if(options & SPANK_SPEED)
+		{
+			print(ML_INFO, NULL, "changing audio speed..");
+			if(forkitf(cb_sox, NULL, SOX_BIN, "sox\n-S\n%s_.wav\n%s.wav\nspeed\n%.9f",
+				p_out->p_filename, p_out->p_filename, p_out->fps / p_in->fps))
+			{
+				print(ML_INFO, NULL, "");
+				printerror("sox failed");
+				freestrings();
+				return 3;
+			}
+			print(ML_INFO, NULL, "");
+			vs_print(p_final, "%s.ac3", p_out->p_filename);
+			unlink(p_final->s);
+			print(ML_INFO, NULL, "converting audio to AC3..");
+			if(forkitf(cb_ffmpeg, p_out, FFMPEG_BIN, "ffmpeg\n-i\n%s.wav\n-acodec\nac3\n-ab\n%d\n-ac\n%d\n%s.ac3",
+				p_out->p_filename, p_out->arate, p_out->achans, p_out->p_filename))
+			{
+				print(ML_INFO, NULL, "");
+				printerror("ffmpeg failed");
+				freestrings();
+				return 3;
+			}
+			print(ML_INFO, NULL, "");
+			vs_print(p_final, "%s.wav", p_out->p_filename);
+			unlink(p_final->s);
+		}
 		print(ML_INFO, NULL, "multiplexing..");
 		if((options & SPANK_SECOND))
 		{
 			if(forkitf(NULL, "mplex", MPLEX_BIN,
-				"mplex -V -f 8 -o %s.mpg %s.m2v %s.ac3",
+				"mplex\n-V\n-f\n8\n-o\n%s.mpg\n%s.m2v\n%s.ac3",
 				p_out->p_filename, p_out->p_filename, p_out->p_filename))
 				{
 					print(ML_INFO, NULL, "");
@@ -506,7 +556,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		else
 		{
 			if(forkitf(NULL, "mplex", MPLEX_BIN,
-				"mplex -f 8 -o %s.mpg %s.m2v %s.ac3",
+				"mplex\n-f\n8\n-o\n%s.mpg\n%s.m2v\n%s.ac3",
 				p_out->p_filename, p_out->p_filename, p_out->p_filename))
 			{
 				print(ML_INFO, NULL, "");
@@ -517,6 +567,11 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		}
 
 		print(ML_INFO, NULL, "");
+		if(options & SPANK_SPEED)
+		{
+			vs_print(p_final, "%s_.wav", p_out->p_filename);
+			unlink(p_final->s);
+		}
 		vs_print(p_final, "%s.m2v", p_out->p_filename);
 		unlink(p_final->s);
 		vs_print(p_final, "%s.ac3", p_out->p_filename);
@@ -547,7 +602,7 @@ int cb_tcprobe(const char *p_string, void *p_options)
 		return 0;
 	}
 
-	if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+	if(pcre_exec(gp_warnerrregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
 		print(ML_WARNING, "tcprobe", "%s", p_string);
 	else
 		print(ML_VERBOSE, "tcprobe", "%s", p_string);
@@ -670,7 +725,7 @@ int cb_transcode(const char *p_string, void *p_options)
 	}
 	else
 	{
-		if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+		if(pcre_exec(gp_warnerrregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
 			print(ML_WARNING, "transcode", "%s", p_string);
 		else
 			print(ML_VERBOSE, "transcode", "%s", p_string);
@@ -679,6 +734,84 @@ int cb_transcode(const char *p_string, void *p_options)
 	return 0;
 }
 
+/* sox callback */
+int cb_sox(const char *p_string, void *p_options)
+{
+	int length;
+	int i;
+	double t[2];
+	int p_vector[30];
+	char p_buffer[1024];
+
+	if(!p_string)
+		return 0;
+
+	length = strlen(p_string);
+	if(length >= sizeof(p_buffer) - 1)
+		length = 1023;
+
+	if(pcre_exec(gp_soxregex, NULL, p_string, length, 0, 0, p_vector,
+		sizeof(p_vector) / sizeof(int)) > 0)
+	{
+		memcpy(p_buffer, p_string, length + 1);
+		p_buffer[length] = 0;
+		for(i = 1; i < 5; i++)
+			p_buffer[p_vector[i * 2 + 1]] = 0;
+
+		t[0] = atof(&p_buffer[p_vector[2]]) * 60.0 + atof(&p_buffer[p_vector[4]]);
+		t[1] = atof(&p_buffer[p_vector[6]]) * 60.0 + atof(&p_buffer[p_vector[8]]);
+
+		fprintf(stderr, "  %.2f%% complete  \r", t[0] * 100.0 / t[1]);
+	}
+	else if(pcre_exec(gp_warnerrregex, NULL, p_string, length, 0, 0, NULL, 0) >= 1)
+	{
+		print(ML_WARNING, "sox", "%s", p_string);
+	}
+	else
+	{
+		print(ML_VERBOSE, "sox", "%s", p_string);
+	}
+	return 0;
+}
+
+/* ffmpeg callback */
+int cb_ffmpeg(const char *p_string, void *p_options)
+{
+	int length;
+	video_t *p_video;
+	int p_vector[10];
+
+	char p_buffer[1024];
+
+	if(!p_string)
+		return 0;
+
+	p_video = (video_t*)p_options;
+
+	length = strlen(p_string);
+	if(length >= sizeof(p_buffer) - 1)
+		length = 1023;
+
+	if(pcre_exec(gp_ffmpegregex, NULL, p_string, length, 0, 0, p_vector,
+		sizeof(p_vector) / sizeof(int)) > 0)
+	{
+		memcpy(p_buffer, p_string, length);
+		p_buffer[length] = 0;
+		p_buffer[p_vector[3]] = 0;
+
+		fprintf(stderr, "  %.2f%% complete  \r", atof(&p_buffer[p_vector[2]]) * 100.0 / p_video->length);
+	}
+	else if(pcre_exec(gp_warnerrregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+	{
+		print(ML_WARNING, "ffmpeg", "%s", p_string);
+	}
+	else
+	{
+		print(ML_VERBOSE, "ffmpeg", "%s", p_string);
+	}
+	return 0;
+}
+
 /* mplayer identify callback */
 int cb_mplayer(const char *p_string, void *p_options)
 {
@@ -695,7 +828,7 @@ int cb_mplayer(const char *p_string, void *p_options)
 	/* option is output to a video_t */
 	p_video = (video_t*)p_options;
 
-	if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+	if(pcre_exec(gp_warnerrregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
 		print(ML_WARNING, "mplayer", "%s", p_string);
 	else
 		print(ML_VERBOSE, "mplayer", "%s", p_string);
@@ -752,7 +885,7 @@ int cb_default(const char *p_string, void *p_options)
 		return 0;
 
 	/* check line against error|warning filter */
-	if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+	if(pcre_exec(gp_warnerrregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
 		print(ML_WARNING, (char*)p_options, "%s", p_string);
 	else
 		print(ML_VERBOSE, (char*)p_options, "%s", p_string);
@@ -770,6 +903,7 @@ int forkit(forkitcallback callback, void *p_options, const char *p_path, const c
 	int retval = 0;
 	int count;
 	int i, pos, start, copy;
+	vstr *p_str;
 
 	cmdlength = strlen(p_cmd);
 	p_cmdline = malloc(cmdlength + 1);
@@ -780,24 +914,30 @@ int forkit(forkitcallback callback, void *p_options, const char *p_path, const c
 	/* create parameter index */
 	strncpy(p_cmdline, p_cmd, cmdlength + 1);
 	pos = start = 0;
-	print(ML_VERBOSE, NULL, "fork %s, %s", p_path, p_cmd);
+
+	p_str = vs_new();
 
 	for(i = 0; i < sizeof(pp_cmdopts) / sizeof(char**); i++)
 	{
-		while(p_cmdline[pos] != ' ' && p_cmdline[pos] != 0)
+		while(p_cmdline[pos] != '\n' && p_cmdline[pos] != 0)
 			pos++;
 
 		pp_cmdopts[i] = &p_cmdline[start];
 		if(p_cmdline[pos] == 0)
 		{
+			vs_cat(p_str, " [%s]", &p_cmdline[start]);
 			pp_cmdopts[i + 1] = 0;
 			break;
 		}
 		p_cmdline[pos] = 0;
+		vs_cat(p_str, " [%s]", &p_cmdline[start]);
 		pos++;
 		start = pos;
 	}
 
+	print(ML_VERBOSE, NULL, "fork %s, %s", p_path, p_str->s);
+	vs_free(p_str);
+
 	/* open comminication pipe */
 	if(pipe(p_pipe) == -1)
 	{
@@ -940,18 +1080,31 @@ int main(int argc, char **argv)
 	if(!(gp_tcvideoregex = pcre_compile("^\\[.*?\\]\\sV:\\s([^ ]*)\\sfps,\\scodec=([^,]*),"
 		"\\sframes=([^,]*),\\swidth=([^,]*),\\sheight=(.*)$", 0, &p_error, &erroffset, NULL)))
 	{
-		printerror("error: gp_tcvideoregex compile failed - %s", p_error);
+		printerror("gp_tcvideoregex compile failed - %s", p_error);
 		exit(1);
 	}
 	if(!(gp_tcaudioregex = pcre_compile("^\\[.*?\\]\\sA:\\s([^ ]*)\\sHz,\\sformat=[^,]*,"
 		"\\sbits=([^,]*),\\schannels=([^,]*),\\sbitrate=(.*),$", 0, &p_error, &erroffset, NULL)))
 	{
-		printerror("error: gp_tcaudioregex compile failed - %s", p_error);
+		printerror("gp_tcaudioregex compile failed - %s", p_error);
 		exit(1);
 	}
-	if(!(gp_warnerrregex = pcre_compile("^.*(?:warning22|error22).*$", PCRE_CASELESS, &p_error, &erroffset, NULL)))
+	if(!(gp_warnerrregex = pcre_compile("^.*(?:warning22|error22).*$",
+		PCRE_CASELESS, &p_error, &erroffset, NULL)))
 	{
-		printerror("error: gp_warnerrregex compile failed - %s", p_error);
+		printerror("gp_warnerrregex compile failed - %s", p_error);
+		exit(1);
+	}
+	if(!(gp_soxregex = pcre_compile("^.*Time:[ ]*(\\d+):(\\d+\\.\\d+) .* of (\\d+):(\\d+\\.\\d+) .*$",
+		0, &p_error, &erroffset, NULL)))
+	{
+		printerror("gp_soxregex compile failed - %s", p_error);
+		exit(1);
+	}
+	if(!(gp_ffmpegregex = pcre_compile("^.* time=(\\d+\\.\\d+) .*$",
+		0, &p_error, &erroffset, NULL)))
+	{
+		printerror("gp_ffmpegregex compile failed - %s", p_error);
 		exit(1);
 	}
 
@@ -1068,6 +1221,9 @@ int main(int argc, char **argv)
 			case 's':
 				calcsize = atoi(optarg);
 				break;
+			case 'S':
+				options |= SPANK_SPEED;
+				break;
 			case 'T':
 				options |= SPANK_PPROCESS;
 				break;
@@ -1153,8 +1309,8 @@ int main(int argc, char **argv)
 
 	/* get source video details */
 	forkitf(cb_mplayer, &invid, MPLAYER_BIN,
-		"mplayer %s -identify -frames 0 -ao null -vo null", invid.p_filename);
-	forkitf(cb_tcprobe, &invid, TCPROBE_BIN, "tcprobe -i %s", invid.p_filename);
+		"mplayer\n%s\n-identify\n-frames\n0\n-ao\nnull\n-vo\nnull", invid.p_filename);
+	forkitf(cb_tcprobe, &invid, TCPROBE_BIN, "tcprobe\n-i\n%s", invid.p_filename);
 	if(!invid.feedback) /* if tcprobe unable to detect, flag use of mplayer for input */
 	{
 		if(invid.p_vcodec[0] == 0)
@@ -1264,33 +1420,44 @@ int main(int argc, char **argv)
 		strcpy(outvid.p_vcodec, "NTSC");
 	}
 
-	/* set length */
-	outvid.length  = invid.length;
-	outvid.hours   = invid.hours;
-	outvid.minutes = invid.minutes;
-	outvid.seconds = invid.seconds;
-	outvid.frames  = invid.frames;
-
 	/* set fps */
 	if(vidmode == 'p' && invid.fps == 25)
 	{
 		outvid.fps = 25;
+		options &= 0xffff ^ SPANK_SPEED;
 	}
 	else if(vidmode =='n' && (invid.fps == 23.976 || invid.fps == 29.97))
 	{
 		outvid.fps = invid.fps;
+		options &= 0xffff ^ SPANK_SPEED;
 	}
 	else if(vidmode == 'p')
 	{
-		options |= SPANK_NEWFPS;
+		if(!(options & SPANK_SPEED))
+			options |= SPANK_HARDFPS;
 		outvid.fps = 25;
 	}
 	else
 	{
-		options |= SPANK_NEWFPS;
+		if(!(options & SPANK_SPEED))
+			options |= SPANK_HARDFPS;
 		outvid.fps = 23.976;
 	}
 
+	/* set length */
+	outvid.frames  = invid.frames;
+	if(outvid.fps == invid.fps || options & SPANK_HARDFPS)
+	{
+		outvid.length = invid.length;
+	}
+	else
+	{
+		outvid.length = ((double)invid.length * invid.fps / outvid.fps);
+	}
+	outvid.hours   = outvid.length / 3600;
+	outvid.minutes = outvid.length % 3600 / 60;
+	outvid.seconds = outvid.length % 60;
+
 	/* set border/crop */
 	if(calcborder)
 	{
@@ -1343,28 +1510,28 @@ int main(int argc, char **argv)
 
 	/* print source and dest video encoding info */
 	print(ML_INFO, NULL, " source | %s", invid.p_filename);
-	print(ML_INFO, NULL,  "  video | %s, %dx%d, %d kbps, %.3f fps",
+	print(ML_INFO, NULL, "  video | %s, %dx%d, %d kbps, %.3f fps",
 		invid.p_vcodec, invid.width, invid.height, invid.vrate, invid.fps);
-	print(ML_INFO, NULL,  "        | %.4f aspect, %d frames, %d:%02d:%02d",
+	print(ML_INFO, NULL, "        | %.4f aspect, %d frames, %d:%02d:%02d",
 		invid.aspect, invid.frames,	invid.hours, invid.minutes, invid.seconds);
-	print(ML_INFO, NULL,  "  audio | %s, %d kbps, %d Hz, %d bit, %d channel",
+	print(ML_INFO, NULL, "  audio | %s, %d kbps, %d Hz, %d bit, %d channel",
 		invid.p_acodec, invid.arate, invid.hz, invid.abits, invid.achans);
-	print(ML_INFO, NULL,  "   file | %d KB", invid.bytes / 1000);
+	print(ML_INFO, NULL, "   file | %d KB", invid.bytes / 1000);
 	if(options & SPANK_MPLAYER)
 		print(ML_INFO, NULL, "   info | using mplayer decoder");
-	print(ML_INFO, NULL,  "");
+	print(ML_INFO, NULL, "");
 
-	print(ML_INFO, NULL,  "   dest | %s.mpg", outvid.p_filename);
-	print(ML_INFO, NULL,  "  video | %s, %dx%d, %d kbps, %.3f fps",
+	print(ML_INFO, NULL, "   dest | %s.mpg", outvid.p_filename);
+	print(ML_INFO, NULL, "  video | %s, %dx%d, %d kbps, %.3f fps",
 		outvid.p_vcodec, outvid.width, outvid.height, outvid.vrate, outvid.fps);
-	print(ML_INFO, NULL,  "        | %.4f aspect, %d frames, %d:%02d:%02d",
+	print(ML_INFO, NULL, "        | %.4f aspect, %d frames, %d:%02d:%02d",
 		outvid.aspect, outvid.frames, outvid.hours, outvid.minutes, outvid.seconds);
-	print(ML_INFO, NULL,  "        | %d(v):%d(h) crop, %s",
+	print(ML_INFO, NULL, "        | %d(v):%d(h) crop, %s",
 		outvid.vcrop, outvid.hcrop, (options & SPANK_MULTI) ? "VBR" : "CBR");
-	print(ML_INFO, NULL,  "  audio | %s, %d kbps, %d Hz, %d bit, %d channel",
+	print(ML_INFO, NULL, "  audio | %s, %d kbps, %d Hz, %d bit, %d channel",
 		outvid.p_acodec, outvid.arate, outvid.hz, outvid.abits, outvid.achans);
-	print(ML_INFO, NULL,  "   file | ~%d KB", outvid.bytes / 1000);
-	print(ML_INFO, NULL,  "");
+	print(ML_INFO, NULL, "   file | ~%d KB", outvid.bytes / 1000);
+	print(ML_INFO, NULL, "");
 
 	exit(encode(&invid, &outvid, options, p_filters));
 }
