@@ -26,7 +26,7 @@
 #define SPANK_NONICE    0x4
 #define SPANK_FIXNTSC   0x8
 #define SPANK_AVPASS    0x10
-#define SPANK_NEWFPS    0x20
+#define SPANK_SPEED     0x20
 #define SPANK_DOUBLEFPS 0x40
 #define SPANK_MULTIPLEX 0x80
 #define SPANK_MPLAYER   0x100
@@ -34,6 +34,7 @@
 #define SPANK_NORMALISE 0x400
 #define SPANK_PPROCESS  0x800
 #define SPANK_AVOLUME   0x1000
+#define SPANK_HARDFPS   0x2000
 
 #ifdef __cplusplus
 extern "C" {
@@ -54,6 +55,8 @@ typedef int (*forkitcallback)(const char *p_parsestring, void *p_options);
 
 void initvideo(video_t *p_video);
 int cb_default(const char *p_string, void *p_options);
+int cb_sox(const char *p_string, void *p_options);
+int cb_ffmpeg(const char *p_string, void *p_options);
 int cb_tcprobe(const char *p_string, void *p_options);
 int cb_transcode(const char *p_string, void *p_options);
 int cb_mplayer(const char *p_string, void *p_options);
